#include "editor.h"
#include "buttons_editor.h"
#include "objects_editor.h"
#include "exa-base.h"
#include "exa-font.h"
#include "exa-image.h"
#include <cmath>
#include <vector>
#include <algorithm>
#include <iostream>
#include <list>
#include <GL/gl.h>
using namespace std;

#define KNOFLIK_X -23
#define KNOFLIK_Y 14.3
#define DIFF 1.0
#define SIZE_KNOFLIK 0.85
#define COLOR_TOOLBAR_SIZEX 23
#define COLOR_KNOFLIK_X -21.5
#define COLOR_KNOFLIK_Y -13.5
#define COLOR_DIFF 0.66
#define COLOR_SIZE_KNOFLIK 0.66
#define MOVE_DIFF 0.02
#define ROTATE_DIFF 0.2
#define SCALE_DIFF_MORE 1.001
#define SCALE_DIFF_LESS 0.999
#define PNG_X_DIFF 47
#define PNG_X2_DIFF 181
#define PNG_Y_DIFF 8
#define PNG_Y2_DIFF 25

color_button::color_button(float r,float g,float b,complex tl,complex br) : red(r), green(g), blue(b), top_left(tl), bottom_right(br)
{}

bool color_button::test_hit(complex screen_pos)
{
	if(screen_pos.x>=top_left.x && screen_pos.x<=bottom_right.x && screen_pos.y>=bottom_right.y && screen_pos.y<=top_left.y)
		return true;
	return false;
}

void color_button::draw()
{
	glColor3f(red,green,blue);
	glBegin(GL_QUADS);
		glVertex2f(top_left.x,bottom_right.y);
		glVertex2f(bottom_right.x,bottom_right.y);
		glVertex2f(bottom_right.x,top_left.y);
		glVertex2f(top_left.x,top_left.y);
	glEnd();
}

void color_button::get_rgb(float& r,float& g,float& b)
{
	r=red;
	g=green;
	b=blue;
}

////////////////////////////////////////////////////////////

color_toolbarx::color_toolbarx()
{
	color_button* cb;
	int i=0;
	for(float r=0;r<=1;r+=0.33)
	{
		for(float g=0;g<=1;g+=0.33)
		{
			for(float b=0;b<=1;b+=0.33)
			{
				cb=new color_button(r,g,b,complex(COLOR_KNOFLIK_X+i*COLOR_DIFF,COLOR_KNOFLIK_Y),complex(COLOR_KNOFLIK_X+i*COLOR_DIFF+COLOR_SIZE_KNOFLIK,COLOR_KNOFLIK_Y-COLOR_SIZE_KNOFLIK));
				colors.push_back(cb);
				++i;
			}
		}
	}
}

color_toolbarx::~color_toolbarx()
{
	list<color_button*>::iterator it=colors.begin();

	while(it!=colors.end())
	{
		delete (*it);
		++it;
	}
}

bool color_toolbarx::check_toolbar(complex screen_pos,float& r,float& g,float& b)
{
	list<color_button*>::iterator it=colors.begin();

	while(it!=colors.end())
	{
		if((*it)->test_hit(screen_pos))
		{
			(*it)->get_rgb(r,g,b);
			return true;
		}
		++it;
	}
	return false;
}

void color_toolbarx::draw_toolbar()
{
	list<color_button*>::iterator it=colors.begin();

	while(it!=colors.end())
	{
		(*it)->draw();
		++it;
	}
}

////////////////////////////////////////////////////////////

toolbarx::toolbarx()
{
	button *b=new select_button(complex(KNOFLIK_X+0*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+0*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/selection.png");
	buttons.push_back(b);
	b=new move_points_button(complex(KNOFLIK_X+2*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+2*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/move_points.png");
	buttons.push_back(b);

	b=new line_button(complex(KNOFLIK_X+4*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+4*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/line.png");
	buttons.push_back(b);
	b=new ellipse_button(complex(KNOFLIK_X+5*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+5*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/ellipse.png");
	buttons.push_back(b);
	b=new rectangle_button(complex(KNOFLIK_X+6*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+6*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/rectangle.png");
	buttons.push_back(b);
	b=new polygon_button(complex(KNOFLIK_X+7*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+7*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/polygon.png");
	buttons.push_back(b);
	b=new ellipse_circle_button(complex(KNOFLIK_X+8*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+8*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/circle.png");
	buttons.push_back(b);
	b=new star_button(complex(KNOFLIK_X+9*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+9*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/star.png");
	buttons.push_back(b);

	b=new makegroup_button(complex(KNOFLIK_X+11*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+11*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/make_group.png");
	buttons.push_back(b);
 b=new destroygroup_button(complex(KNOFLIK_X+12*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+12*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/destroy_group.png");
	buttons.push_back(b);
	b=new copy_button(complex(KNOFLIK_X+13*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+13*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/copy.png");
	buttons.push_back(b);
	b=new remove_button(complex(KNOFLIK_X+14*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+14*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/cut.png");
	buttons.push_back(b);

	b=new raise_top_button(complex(KNOFLIK_X+16*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+16*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/raise_top.png");
	buttons.push_back(b);
	b=new raise_step_button(complex(KNOFLIK_X+17*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+17*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/raise_layer.png");
	buttons.push_back(b);
	b=new lower_step_button(complex(KNOFLIK_X+18*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+18*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/lower_layer.png");
	buttons.push_back(b);
	b=new lower_bottom_button(complex(KNOFLIK_X+19*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+19*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/lower_bottom.png");
	buttons.push_back(b);

	b=new backward_button(complex(KNOFLIK_X+21*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+21*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/undo.png");
	buttons.push_back(b);
	b=new forward_button(complex(KNOFLIK_X+22*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+22*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/forward.png");
	buttons.push_back(b);
	b=new screen_shot_button(complex(KNOFLIK_X+24*DIFF,KNOFLIK_Y+0),complex(KNOFLIK_X+24*DIFF+SIZE_KNOFLIK,KNOFLIK_Y-SIZE_KNOFLIK),"img/screen_shot.png");
	buttons.push_back(b);

	active_button=0;
	text.loadfromfreetype("verdana.ttf");
}

toolbarx::~toolbarx()
{
	vector<button*>::iterator it=buttons.begin();

	while(it!=buttons.end())
	{
		delete (*it);
		++it;
	}
}

void toolbarx::do_action(vector<complex>& p, view* v)
{
	buttons[active_button]->do_action(v->action,v);
}

void toolbarx::draw_toolbar()
{
	vector<button*>::iterator it=buttons.begin();

	while(it!=buttons.end())
	{
		(*it)->draw();
		++it;
	}
}

void toolbarx::draw_tooltip(complex screen_pos, const char *s)
{
	glPushMatrix();
	glEnable(GL_BLEND); glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glTranslatef(screen_pos.x, screen_pos.y-1.1,0.1);
	glScalef(0.7,0.7,0);
	glColor3f(1,1,1);
	text.write(s);
	glDisable(GL_BLEND);
	glPopMatrix();
}

int toolbarx::check_toolbar(complex screen_pos)
{
	vector<button*>::iterator it=buttons.begin();
	int i=0;
		
	while(it!=buttons.end())
	{
		if((*it)->top_left.x<=screen_pos.x && (*it)->top_left.y>=screen_pos.y && (*it)->bottom_right.x>=screen_pos.x && (*it)->bottom_right.y<=screen_pos.y)
			return i;
		++it;
		++i;
	}
	return -1;
}

void toolbarx::check_button_tooltips(complex screen_pos)
{
	vector<button*>::iterator it=buttons.begin();
	int i=0;
		
	while(it!=buttons.end())
	{
		if((*it)->is_being_pointed(screen_pos))
		{
			draw_tooltip(screen_pos,buttons[i]->get_name());
			break;
		}
		++it;
		++i;
	}
}

void toolbarx::erase_resize_points_memory()
{
	move_points_button* p=dynamic_cast<move_points_button*>(buttons[1]);
	if(p)
		p->erase_resize_points_memory();
}

bool toolbarx::is_making_snapshot()
{
	if(active_button==18)
		return true;
	return false;
}

////////////////////////////////////////////////////////////

button::button(complex tl, complex br, const char* s) : top_left(tl), bottom_right(br)
{
	int x,y,b;
	char*d;
	if(exaImageReadPNG(s,&x,&y,&b,&d))
	{
		image=exaImageMakeGLTex(x,y,b,d);
		exaImageFree(d);
	}
	else
		cout<<" problem s texturou\n";
}

void button::draw()
{
	glEnable(GL_TEXTURE_2D);
	glColor3f(1,1,1);
	glBindTexture(GL_TEXTURE_2D,image);
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(top_left.x,bottom_right.y);
		glTexCoord2f(1,0);
		glVertex2f(bottom_right.x,bottom_right.y);
		glTexCoord2f(1,1);
		glVertex2f(bottom_right.x,top_left.y);
		glTexCoord2f(0,1);
		glVertex2f(top_left.x,top_left.y);
	glEnd();
	glDisable(GL_TEXTURE_2D);
}

bool button::is_being_pointed(complex screen_pos)
{
	if(screen_pos.x>=top_left.x && screen_pos.x<=bottom_right.x && screen_pos.y>=bottom_right.y && screen_pos.y<=top_left.y)
		return true;
	return false;
}

button::~button()
{
	exaImageDelGLTex(image);
}

////////////////////////////////////////////////////////////

bool already_selected(objekt *ob, list<objekt*> &ob_uk_list)
{
	list<objekt*>::iterator it;
	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		if((*it)==ob) return true;
		++it;
	}
	return false;
}

void select_button::do_action(vector<complex>& p, view* v)
{
	list<objekt*> ob_uk_list;
	v->selection.get_o(ob_uk_list);
	bool vycistit=true;

	if(p.size() && v->selection.test_arrow_hit(p[0])!=(-1))
		p.clear();
	if(p.size())
	{
		float r,g,b;
		if(v->color_toolbar.check_toolbar((p[0]+v->camera_position)*v->zoom,r,g,b))
		{
			objekt* ob_uk;
			list<objekt*> ob_uk_list;
			v->selection.get_o(ob_uk_list);
			list<objekt*> ob_uk_list2;
			v->canvas.get_o(ob_uk_list2);

			v->selection.clean();
			list<objekt*>::iterator it;
			it=ob_uk_list.begin();
			while(it!=ob_uk_list.end())
			{
				ob_uk=(*it)->duplicate();		//delete "old" pointers on objects; and replace them by pointers on their exact copies
				replace(ob_uk_list2.begin(),ob_uk_list2.end(),*it,ob_uk);
				v->selection.add_obj(ob_uk);
				++it;
			}
			v->canvas.change(ob_uk_list2);

			v->selection.change_color_absolutely(r,g,b);
			v->change_color_absolutely(r,g,b);
			p.clear();
			return;
		}
		objekt* ob=v->canvas.get_pointed_object(p[0]);
		if(!ob)
		{
			if(!exaIsKeyDown(EKEY_LSHIFT) && !exaIsKeyDown(EKEY_RSHIFT))
			{
				v->selection.clean();
				v->selection.rotating=false;
			}
		}
		else
		{
			if(!exaIsKeyDown(EKEY_LSHIFT) && !exaIsKeyDown(EKEY_RSHIFT) &&find(ob_uk_list.begin(),ob_uk_list.end(),ob)!=ob_uk_list.end() && p[0]==v->cursor)
			{
				if(exaIsMouseButtonDown(1))
					vycistit=false;
				else
				{
					if(v->selection.rotating)
						v->selection.rotating=false;
					else
						v->selection.rotating=true;
				}
			}
			else
			{
				if(!exaIsKeyDown(EKEY_LSHIFT) && !exaIsKeyDown(EKEY_RSHIFT))
				{
					if(!already_selected(ob, ob_uk_list))
					{
						v->selection.clean();
						v->selection.add_obj(ob);
						v->selection.rotating=false;
					}
				}
				else
				{
					if(already_selected(ob, ob_uk_list) && p[0]==v->cursor) 
					{
						if(exaIsMouseButtonDown(1))
							vycistit=false;
						else
						{
							v->selection.rem_obj(ob);
							v->selection.rotating=false;
						}
					}
					else
					{
						v->selection.add_obj(ob);
						v->selection.rotating=false;
					}
				}
			}
		}
		if(vycistit) 
			p.clear();
	}

	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.empty())
	{
		list<objekt*>::iterator it;
		it=ob_uk_list.begin();
		v->selection.draw_selection(v->zoom);

		if(exaIsKeyDown(EKEY_r) && exaGetElapsedTime()>0.07)
		{
			it=ob_uk_list.begin();
			list<objekt*> ob_uk_list2;
			v->canvas.get_o(ob_uk_list2);
			objekt_group temp;
			list<objekt*> temp_ob_uk_list;
			temp.put_o(ob_uk_list2);
			while(it!=ob_uk_list.end())
			{
					temp.rem_obj(*it);
					++it;
			}
			temp.get_o(temp_ob_uk_list);
			v->canvas.change(temp_ob_uk_list);
			v->selection.clean();
			temp.clean();
		}
		if(exaIsKeyDown(EKEY_UP) || exaIsKeyDown(EKEY_DOWN) || exaIsKeyDown(EKEY_RIGHT) || exaIsKeyDown(EKEY_LEFT))
		{
			if(exaIsKeyDown(EKEY_UP))
				v->selection.move(complex(0,MOVE_DIFF));
			if(exaIsKeyDown(EKEY_DOWN))
				v->selection.move(complex(0,-MOVE_DIFF));
			if(exaIsKeyDown(EKEY_RIGHT))
				v->selection.move(complex(MOVE_DIFF,0));
			if(exaIsKeyDown(EKEY_LEFT))
				v->selection.move(complex(-MOVE_DIFF,0));
		}
		if(exaIsKeyDown(EKEY_PAGEUP) || exaIsKeyDown(EKEY_PAGEDOWN))
		{
			complex tl,br,stred;
			v->selection.get_bounding_box(tl,br);
			stred=(tl+br)/2;
			if(exaIsKeyDown(EKEY_PAGEUP))
				v->selection.rotate(stred,ROTATE_DIFF);
			if(exaIsKeyDown(EKEY_PAGEDOWN))
				v->selection.rotate(stred,-ROTATE_DIFF);
		}
		if(exaIsKeyDown(EKEY_HOME) || exaIsKeyDown(EKEY_END))
		{
			complex tl,br,stred;
			v->selection.get_bounding_box(tl,br);
			stred=(tl+br)/2;
			if(exaIsKeyDown(EKEY_HOME))
				v->selection.scale(stred,SCALE_DIFF_MORE);
			if(exaIsKeyDown(EKEY_END))
				v->selection.scale(stred,SCALE_DIFF_LESS);
		}
		if(exaIsKeyDown(EKEY_1) || exaIsKeyDown(EKEY_2) || exaIsKeyDown(EKEY_3))
		{
			float pom=1;
			if(exaIsKeyDown(EKEY_LSHIFT) || exaIsKeyDown(EKEY_RSHIFT))
				pom=-1;
			if(exaIsKeyDown(EKEY_1))
			{
				v->selection.change_color(pom/500,0,0);
				v->change_color(pom/500,0,0);
			}
			if(exaIsKeyDown(EKEY_2))
			{
				v->selection.change_color(0,pom/500,0);
				v->change_color(0,pom/500,0);
			}
			if(exaIsKeyDown(EKEY_3))
			{
				v->selection.change_color(0,0,pom/500);
				v->change_color(0,0,pom/500);
			}
		}
	}
}

void makegroup_button::do_action(vector<complex>& p, view* v)
{
	list<objekt*> ob_uk_list;
	v->selection.get_o(ob_uk_list);
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);
	v->toolbar.active_button=0;

	if(ob_uk_list.size()>1)
	{
		list<objekt*>::iterator it;
		it=ob_uk_list2.begin();
		objekt_group *g=new objekt_group;
		while(it!=ob_uk_list2.end())
		{
			if(find(ob_uk_list.begin(),ob_uk_list.end(),*it)!=ob_uk_list.end())
			{
				g->add_obj((*it)->duplicate());
				temp.rem_obj(*it);
			}
			++it;
		}
		temp.add_obj(g);
		temp.get_o(temp_ob_uk_list);
		v->canvas.change(temp_ob_uk_list);
		v->selection.clean();
		v->selection.add_obj(g);
	}
	temp.clean();
}

void destroygroup_button::do_action(vector<complex>& p, view* v)
{
	list<objekt*> ob_uk_list;
	v->selection.get_o(ob_uk_list);
	v->toolbar.active_button=0;

	if(ob_uk_list.size()==1)
	{
		objekt* pointer;
		list<objekt*>::iterator it;
		it=ob_uk_list.begin();
		objekt_group *ob=dynamic_cast<objekt_group *> (*it);
		if(!ob) return;

		ob->get_o(ob_uk_list);
		it=ob_uk_list.begin();

		v->selection.clean();

		list<objekt*> ob_uk_list2;
		v->canvas.get_o(ob_uk_list2);
		objekt_group temp;
		list<objekt*> temp_ob_uk_list;
		temp.put_o(ob_uk_list2);
		while(it!=ob_uk_list.end())
		{
			pointer=(*it)->duplicate();
			v->selection.add_obj(pointer);
			temp.add_obj(pointer);
			++it;
		}
		temp.rem_obj(ob);
		temp.get_o(temp_ob_uk_list);
		v->canvas.change(temp_ob_uk_list);
		temp.clean();
	}
}

void copy_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	objekt* ob;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	v->selection.clean();
		list<objekt*> ob_uk_list2;
		v->canvas.get_o(ob_uk_list2);
	it=ob_uk_list2.begin();
		objekt_group temp;
		list<objekt*> temp_ob_uk_list;
		temp.put_o(ob_uk_list2);
	while(it!=ob_uk_list2.end())
	{
		if(find(ob_uk_list.begin(),ob_uk_list.end(),*it)!=ob_uk_list.end())
		{
			ob=(*it)->duplicate();
			temp.add_obj(ob);
			v->selection.add_obj(ob);
		}
		++it;
	}
		temp.get_o(temp_ob_uk_list);
		v->canvas.change(temp_ob_uk_list);
		temp.clean();
}

void remove_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	list<objekt*> ob_uk_list;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	list<objekt*>::iterator it;
	it=ob_uk_list.begin();
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);
	while(it!=ob_uk_list.end())
	{
			temp.rem_obj(*it);
			++it;
	}
	temp.get_o(temp_ob_uk_list);
	v->canvas.change(temp_ob_uk_list);
	v->selection.clean();
	temp.clean();
}

void polygon_button::do_action(vector<complex>& p, view* v)
{
	if(!exaIsKeyDown(EKEY_LSHIFT) && !exaIsKeyDown(EKEY_RSHIFT))
	{
		if(p.size()<3)
			p.clear();
		else
		{
			v->canvas.add_obj(new polygon(p,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
	}
	else
	{
		if(p.size())
		{
			vector<complex>::iterator it=p.begin();
			glColor3f(v->last_red, v->last_green, v->last_blue);
			glBegin(GL_LINE_STRIP);
				while(it!=p.end())
				{
					glVertex2f(it->x,it->y);
					++it;
				}
				glVertex2f(v->cursor.x,v->cursor.y);
			glEnd();
		}
	}
}

void line_button::do_action(vector<complex>& p, view* v)
{
	if(!exaIsKeyDown(EKEY_LSHIFT) && !exaIsKeyDown(EKEY_RSHIFT))
	{
		if(p.size()>1)
		{
			v->canvas.add_obj(new line(p,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
	}
	if(p.size())
	{
		vector<complex>::iterator it=p.begin();
		glColor3f(v->last_red, v->last_green, v->last_blue);
		glBegin(GL_LINE_STRIP);
			while(it!=p.end())
			{
				glVertex2f(it->x,it->y);
				++it;
			}
			glVertex2f(v->cursor.x,v->cursor.y);
		glEnd();
	}
}

void circle_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>1)
	{
		v->canvas.add_obj(new circle(p[0],p[1],v->last_red,v->last_green,v->last_blue));
		p.clear();
	}
}

void rectangle_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>0)
	{
		if(p[0]==v->cursor && !exaIsMouseButtonDown(1)) 
		{
			p.clear();
			return;
		}
		if(p[0]==v->cursor) 
			return;

		vector<complex> points;
		points.push_back(p[0]);
		points.push_back(complex(p[0].x,v->cursor.y));
		points.push_back(v->cursor);
		points.push_back(complex(v->cursor.x,p[0].y));

		if(!exaIsMouseButtonDown(1))
		{
			v->canvas.add_obj(new rectangle(points,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
		else
		{
			rectangle rec(points,v->last_red,v->last_green,v->last_blue);
			rec.draw();
		}
	}
}

void ellipse_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>0)
	{
		if(p[0]==v->cursor && !exaIsMouseButtonDown(1)) 
		{
			p.clear();
			return;
		}
		if(p[0]==v->cursor) 
			return;

		vector<complex> points;
		float xrad=fabsf((p[0].x - v->cursor.x)/2);
		float yrad=fabsf((p[0].y - v->cursor.y)/2);
		complex middle((p[0] + v->cursor)/2);
		for(int i=0; i <= 360; i++)
			points.push_back(complex(middle.x+(cos(i*M_PI/180)*xrad),middle.y+(sin(i*M_PI/180)*yrad)));

		if(!exaIsMouseButtonDown(1))
		{
			v->canvas.add_obj(new ellipse(points,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
		else
		{
			ellipse el(points,v->last_red,v->last_green,v->last_blue);
			el.draw();
		}
	}
}

void ellipse_circle_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>0)
	{
		if(p[0]==v->cursor && !exaIsMouseButtonDown(1)) 
		{
			p.clear();
			return;
		}
		if(p[0]==v->cursor) 
			return;

		vector<complex> points;
		complex middle((p[0] + v->cursor)/2);
		float xrad=(fabsf((p[0].x - v->cursor.x)/2) + fabsf((p[0].y - v->cursor.y)/2))/2;
		for(int i=0; i <= 360; i++)
			points.push_back(complex(middle.x+(cos(i*M_PI/180)*xrad),middle.y+(sin(i*M_PI/180)*xrad)));

		if(!exaIsMouseButtonDown(1))
		{
			v->canvas.add_obj(new ellipse_circle(points,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
		else
		{
			ellipse_circle el(points,v->last_red,v->last_green,v->last_blue);
			el.draw();
		}
	}
}

void star_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>0)
	{
		if(p[0]==v->cursor && !exaIsMouseButtonDown(1)) 
		{
			p.clear();
			return;
		}
		if(p[0]==v->cursor) 
			return;

		vector<complex> points;
		complex middle((p[0] + v->cursor)/2);
		float r((p[0] - middle).length());
		float r2=r/2;

		points.push_back(middle);
		for(int i=0;i<5;i++)
		{
			points.push_back(complex(middle.x+r*cos((i*72+90)*M_PI/180),middle.y+r*sin((i*72+90)*M_PI/180)));
			points.push_back(complex(middle.x+r2*cos((i*72+126)*M_PI/180),middle.y+r2*sin((i*72+126)*M_PI/180)));
		}

		if(!exaIsMouseButtonDown(1))
		{
			v->canvas.add_obj(new star(points,v->last_red,v->last_green,v->last_blue));
			p.clear();
		}
		else
		{
			star st(points,v->last_red,v->last_green,v->last_blue);
			st.draw();
		}
	}
}

void raise_step_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);

	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		temp.raise_step(*it);
		++it;
	}

	temp.get_o(temp_ob_uk_list);
	v->canvas.change(temp_ob_uk_list);
	temp.clean();
}

void raise_top_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);

	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		temp.raise_top(*it);
		++it;
	}
	temp.get_o(temp_ob_uk_list);
	v->canvas.change(temp_ob_uk_list);
	temp.clean();
}

void lower_step_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);

	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		temp.lower_step(*it);
		++it;
	}
	temp.get_o(temp_ob_uk_list);
	v->canvas.change(temp_ob_uk_list);
	temp.clean();
}

void lower_bottom_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;
	v->selection.get_o(ob_uk_list);
	if(!ob_uk_list.size())
		return;
	list<objekt*> ob_uk_list2;
	v->canvas.get_o(ob_uk_list2);
	objekt_group temp;
	list<objekt*> temp_ob_uk_list;
	temp.put_o(ob_uk_list2);

	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		temp.lower_bottom(*it);
		++it;
	}
	temp.get_o(temp_ob_uk_list);
	v->canvas.change(temp_ob_uk_list);
	temp.clean();
}

void backward_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	v->selection.clean();
	v->canvas.backward();
}

void forward_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	v->selection.clean();
	v->canvas.forward();
}

void screen_shot_button::do_action(vector<complex>& p, view* v)
{
	v->toolbar.active_button=0;
	time_t t;
	t = time(0);
	string a=ctime(&t);
	a.replace(a.size()-1,1,".png");
	a="snapshots/"+a;
	int x,y,bytes;
	char* d;
	if(this->exaScreenshot(v->png_image_coords.startx+PNG_X_DIFF,v->png_image_coords.starty+PNG_Y_DIFF,v->png_image_coords.endx+PNG_X2_DIFF,v->png_image_coords.endy+PNG_Y2_DIFF,&x,&y,&bytes,&d))
	{
		exaImageWritePNG(a.c_str(), x,  y, bytes, d);
	}
}

bool screen_shot_button::exaScreenshot(int startx, int starty, int endx, int endy, int*x, int*y, int*bytes, char**d)
{
	char*data;

	data=(char*)malloc((endx-startx)*(endy-starty)*3);
	if(!data)return false;
	glPixelStorei(GL_PACK_ALIGNMENT,1);
	glReadPixels(startx,starty,(endx-startx),(endy-starty),0?GL_BGR:GL_RGB,GL_UNSIGNED_BYTE,data);
	*x=endx-startx;
	*y=endy-starty;
	*bytes=3;
	*d=data;
	return true;
}

void move_points_button::do_action(vector<complex>& p, view* v)
{
	if(p.size()>0)
	{
		persist_moving=false;
		if(po)		//test, whether "dragging point" was clicked
		{
			if(po->test_hit_drag_points(p[0],&point,v->zoom))
			{
				persist_moving=true;
			}
		}
		if(!persist_moving)
		{
			objekt* ob;
			ob=v->canvas.get_pointed_object_concrete(p[0]);
			po=dynamic_cast<poly_objekt*> (ob);
		}
		p.clear();
	}
	if(po)
	{
		po->draw_resizeable_points(v->zoom);
		if(exaIsMouseButtonDown(1))
		{
			if(persist_moving && (v->from_pom!=v->to))
			{
				if(v->from==v->from_pom)  //moved for the first time - we have to save current position of objects (in selection) being moved
				{
					list<objekt*> ob_uk_list;
					v->canvas.get_o(ob_uk_list);
					objekt_group temp;
					list<objekt*> temp_ob_uk_list;
					temp.put_o(ob_uk_list);
					objekt_group* ob_g_uk=dynamic_cast<objekt_group*> (temp.duplicate(&po));
					if(!ob_g_uk || !po)
					{
						temp.clean();
						return;
					}
					po->test_hit_drag_points(*point,&point,v->zoom);
					ob_g_uk->get_o(temp_ob_uk_list);
					v->canvas.change(temp_ob_uk_list);
					temp.clean();
				}
				po->move_point(point,(v->to - v->from_pom));
				v->from_pom=v->to;
			}
		}
		else
		{
			persist_moving=false;
		}
	}
}

void move_points_button::erase_resize_points_memory()
{
	po=0;
}
