#ifndef _BUTTONS_EDITOR
#define _BUTTONS_EDITOR

#include "exa-font.h"
#include <list>
#include <vector>
#include "complex.h"
#include "editor.h"
#include "objects_editor.h"

class button;
class view;

/**predstavuje tzv. farebny cudlik, po stlaceni ktoreho objekty v selection zmenia svoju na prislusnu stlacenu farbu*/
class color_button
{
	/**cervena zlozka farby cudlika*/
	float red;
	/**zelena zlozka farby cudlika*/
	float green;
	/**modra zlozka farby cudlika*/
	float blue;
	/**lavy horny roh cudlika*/
	complex top_left;
	/**pravy dolny roh cudlika*/
	complex bottom_right;
public:
	color_button(float,float,float,complex,complex);
	/**testujeme, ci bol farebny cudlik stlaceny*/
	bool test_hit(complex);
	/**vykresli cudlik*/
	void draw();
	/**prostrednictvom parametrov cudlik preda svoju farbu*/
	void get_rgb(float&,float&,float&);
};

/**trieda, ktora zdruzuje farebne cudliky*/
class color_toolbarx
{
	/**list obsahujuci farebne cudliky*/
	list<color_button*> colors;
public:
	color_toolbarx();
	~color_toolbarx();
	/**metoda zisti, ci bolo kliknute na niektore z cudlikov. Ak ano, tak vrati farbu (prostrednictvom parametrov), na ktoru bolo kliknute*/
	bool check_toolbar(complex screen_pos,float&,float&,float&);
	/**vykresli vsetky farebne cudliky*/
	void draw_toolbar();
};

/**trieda, ktora zdruzuje ovladacie cudliky*/
class toolbarx
{
	/**udrzujeme si "tlacitka" na obsluhu ovladania*/
	vector<button*> buttons; 
	/**premenna, ktoru vyuziva openGl na zobrazovanie textu - tooltipy, ktore sa objavuju ako popisky cudlikov, ked kurzorom prejdeme cez cudliky*/
	exaGLFont text;
public:
	toolbarx();
	~toolbarx();
	/**oznacuje ktory "knoflik" v nasom toolbare je prave aktivny*/
	int active_button;
	/**metoda, ktora obsluhuje prave aktivny cudlik
	 * \note vo svojom tele potom vola vykonnu metodu aktivneho cudliku*/
	void do_action(vector<complex>& p, view* v);
	/**zisti, ci nebola klliknute na niektory z cudlikov*/
	int check_toolbar(complex screen_pos);
	/**vykresli vsetky cudliky*/
	void draw_toolbar();
	/**zisti, ci sa kurzor nenachadza nad nejakym cudlikom*/
	void check_button_tooltips(complex screen_pos);
	/**ak sa kurzor nachadza nad nejakym cudlikom, tak nad nim vykresli popisok - tooltip*/
	void draw_tooltip(complex screen_pos, const char *s);
	/**v pripade aktivneho cudliku, ktory posuva nejaky bod objektu typu poly_objekt, vynuluje ukazatel na ten poly_objekt, ked uz jeho bod nie je posuvany*/
	void erase_resize_points_memory();
	/**vracia true, ak sa prave bude ukladat obrazok do png*/
	bool is_making_snapshot();
};

/**otcovska trieda "cudlikov"*/
class button
{
	/**openGl premenna na ukladanie textur - obrazkov cudlikov*/
	GLuint image;
public:
	/**lavy horny roh cudlika*/
	complex top_left;
	/**pravy dolny roh cudlika*/
	complex bottom_right;
	button(complex tl, complex br, const char* s);
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	virtual const char* get_name()=0;
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p,view* v)=0;
	/**vykresli sa cudlik*/
	virtual void draw();
	/**testuje, ci na cudlik bolo kliknute*/
	bool is_being_pointed(complex screen_pos);
	virtual ~button();
};

/**uloha tohto cudliku je oznacovat objekty, presuvat ich, zmensovat/zvacsovat a rotovat objekty*/
class select_button : public button
{
public:
	select_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "select";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je zdruzit niekolko oznacenych objektov do jedneho*/
class makegroup_button : public button
{
public:
	makegroup_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "make group";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je vytvorit kopie oznacenych objektov*/
class copy_button : public button
{
public:
	copy_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "make copy";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je odstranit oznacene objekty*/
class remove_button : public button
{
public:
	remove_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "remove selected objects";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je rozdelit objekt, ktory predtym vznikol spojenim viacerych objektov, naspat na povodne jednotlive objekty*/
class destroygroup_button : public button
{
public:
	destroygroup_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "destroy group";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit mnohouholniky
 * \note pri kresleni je potrebne drzat klavesu shift*/
class polygon_button : public button
{
public:
	polygon_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw polygon";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit usecky alebo spojene usecky s drzanim klavesy shift*/
class line_button : public button
{
public:
	line_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw line";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit kruhy*/
class circle_button : public button
{
public:
	circle_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw circle";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit obdlzniky*/
class rectangle_button : public button
{
public:
	rectangle_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw rectangular";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit elipsy*/
class ellipse_button : public button
{
public:
	ellipse_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw ellipse";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit kruhy, ktore sa avsak rovnako spravaju ako elipsy*/
class ellipse_circle_button : public button
{
public:
	ellipse_circle_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw circle";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je kreslit hviezicky*/
class star_button : public button
{
public:
	star_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "draw star";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je oznaceny objekt presunut blizsie do popredia o jeden krok (ak samozrejme sa pred nim nachadza nejaky iny objekt)*/
class raise_step_button : public button
{
public:
	raise_step_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "raise selection one layer";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je oznaceny objekt presunut dalej do pozadia o jeden krok (ak samozrejme sa za nim nachadza nejaky iny objekt)*/
class lower_step_button : public button
{
public:
	lower_step_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "lower selection one layer";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je oznaceny objekt presunut uplne do popredia (ak samozrejme sa pred nim nachadza nejaky iny objekt)*/
class raise_top_button : public button
{
public:
	raise_top_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "raise selection to top";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je oznaceny objekt presunut uplne do pozadia (ak samozrejme sa za nim nachadza nejaky iny objekt)*/
class lower_bottom_button : public button
{
public:
	lower_bottom_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "lower selection to bottom";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je prevedenie undo (krok dozadu)*/
class backward_button : public button
{
public:
	backward_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "backward";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je vratit undo (krok dopredu)*/
class forward_button : public button
{
public:
	forward_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "forward";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
};

/**uloha tohto cudliku je ulozit vytvoreny objekty do obrazku v png formate*/
class screen_shot_button : public button
{
public:
	screen_shot_button(complex tl, complex br, const char* s): button(tl,br,s) {}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "save image in png format";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
	/**pomocna funkcia, ktora prostrednictvom parametru vracia pole farieb specifikovaneho obdlznika*/
	bool exaScreenshot(int startx, int starty, int endx, int endy, int*x, int*y, int*bytes, char**d);
};

/**uloha tohto cudliku je presuvat jednodlive body objektu typu poly_objekt*/
class move_points_button : public button
{
	/**ukazatel na objekt, ktoreho bod budeme presuvat*/
	poly_objekt* po;
	/**ukazuje na konkretny bod, ktory je posuvany*/
	complex* point;
	/**indikuje stav, kedy mame nadalej pokracovat s presuvanim bodu*/
	bool persist_moving;
public:
	move_points_button(complex tl, complex br, const char* s): button(tl,br,s) {po=0;persist_moving=false;}
	/**vracia nazov cudlika - tooltip (popisok) cudlika, ktory sa zobrazuje pri umiestneni kurzoru nad cudlikom*/
	const char* get_name() { return "move nodes of objects";}
	/**podnikne akciu prisluchajucu k danemu cudliku*/
	virtual void do_action(vector<complex>& p, view* v);
	/**vynuluje ukazatel na objekt, ktoreho bod uz nie je posuvany*/
	void erase_resize_points_memory();
};

#endif
