#include "editor.h"
#include "exa-base.h"
#include <cmath>
#include <algorithm>
#include <iostream>
#include <list>
#include <GL/gl.h>
using namespace std;

#define SIZE_ARROW 0.5
#define CANVAS_SIZE 50
#define ZOOM_IN_MULTIPLIER 1.11111
#define ZOOM_OUT_MULTIPLIER 0.9
#define ZOOM_MAX 100
#define ZOOM_MIN 0.05
#define RESIZE_MIN 0.03
#define SCREEN_X_PIXELS 1024
#define SCREEN_Y_PIXELS 768
#define SCREEN_X 23
#define SCREEN_Y 14.5

canvas_group::canvas_group()
{
	active=diff=0;
	size=CANVAS_SIZE;
	can=new objekt_group[size];
}

canvas_group::~canvas_group()
{
	int i,j;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;

	for(i=0;i<size;++i)
	{
		can[i].get_o(ob_uk_list);
		it=ob_uk_list.begin();
		while(it!=ob_uk_list.end())
		{
			for(j=i+1;j<size;++j)
			{
				can[j].rem_obj(*it);
			}
			++it;
		}
	}
	delete [] can;
}

void canvas_group::overwrite(int position)
{
	int i;
	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;

	can[position].get_o(ob_uk_list);
	it=ob_uk_list.begin();
	while(it!=ob_uk_list.end())
	{
		for(i=0;i<size;++i)
		{
			if(i==position)
				continue;
			if(can[i].member(*it))
				break;
		}
		if(i==size)
			delete *it;
		++it;
	}
}

void canvas_group::draw()
{
	can[active].draw();
}
void canvas_group::change(list<objekt*>& ob_list)
{
	++active;
	if(active>size-1)
		active=0;
	overwrite(active);
	can[active].put_o(ob_list);
	diff=0;
}
void canvas_group::get_o(list<objekt*>& ob_list)
{
	can[active].get_o(ob_list);
}
void canvas_group::rem_obj(objekt* ob)
{	
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].rem_obj(ob);
	active=succ;
	diff=0;
}
void canvas_group::add_obj(objekt* ob)
{
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].add_obj(ob);
	active=succ;
	diff=0;
}
void canvas_group::raise_step(objekt* ob)
{
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].raise_step(ob);
	active=succ;
	diff=0;
}
void canvas_group::raise_top(objekt* ob)
{
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].raise_top(ob);
	active=succ;
	diff=0;
}
void canvas_group::lower_step(objekt* ob)
{
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].lower_step(ob);
	active=succ;
	diff=0;
}
void canvas_group::lower_bottom(objekt* ob)
{
	int succ=active+1;
	if(succ==size)
		succ=0;
	overwrite(succ);
	list<objekt*> ob_list;
	can[active].get_o(ob_list);
	can[succ].put_o(ob_list);
	can[succ].lower_bottom(ob);
	active=succ;
	diff=0;
}

void canvas_group::backward()
{
	if(diff<size-1)
		++diff;
	else
		return;

	--active;
	if(active<0)
		active=size-1;
}

void canvas_group::forward()
{
	if(!diff)
		return;
	--diff;

	++active;
	if(active>size-1)
		active=0;
}

objekt* canvas_group::get_pointed_object(complex screen_pos)
{
	list<objekt*> ob_list;
	can[active].get_o(ob_list);

	list<objekt*>::iterator it=ob_list.end();
		
	while(it!=ob_list.begin())
	{
		--it;
		if((*it)->test_hit(screen_pos))
			return *it;
	}
	return 0;
}

objekt* canvas_group::get_pointed_object_concrete(complex screen_pos)
{
	return (can[active].test_hit_concrete(screen_pos));
}

//////////////////////////////////////////////////////////////////

void arrow::set_coords(complex tl, complex br)
{
	top_left=tl;
	bottom_right=br;
}

void arrow::draw(bool rotate)
{
	if(rotate)
		glColor3f(0,1,1);
	else
		glColor3f(1,1,0);
		
	glBegin(GL_POLYGON);
		glVertex2f(top_left.x,top_left.y);
		glVertex2f(bottom_right.x,top_left.y);
		glVertex2f(bottom_right.x,bottom_right.y);
		glVertex2f(top_left.x,bottom_right.y);
	glEnd();
}

bool arrow::test_hit(complex cursor)
{
	if((cursor.x >= top_left.x) && (cursor.y <= top_left.y) && (cursor.x <= bottom_right.x) && (cursor.y >= bottom_right.y))
		return true;
	else 
		return false;
}

//////////////////////////////////////////////////////////////////

selection_group::selection_group()
{
	arrows=new arrow[8];
	rotating=false;
}

selection_group::~selection_group()
{	
	delete [] arrows;
}

void selection_group::draw_arrows()
{
	for(int i=0;i<8;++i)
		arrows[i].draw(this->rotating);
}

void selection_group::draw_selection(float zoom)
{
	check_bounding_box();
	glColor3f(0.5,0.5,0.5);
	glBegin(GL_LINE_LOOP);
		glVertex2f(top_left.x,top_left.y);
		glVertex2f(bottom_right.x,top_left.y);
		glVertex2f(bottom_right.x,bottom_right.y);
		glVertex2f(top_left.x,bottom_right.y);
	glEnd();

	arrows[0].set_coords(complex(top_left.x-SIZE_ARROW/zoom,top_left.y+SIZE_ARROW/zoom),complex(top_left.x,top_left.y));

	arrows[1].set_coords(complex((top_left.x+bottom_right.x-SIZE_ARROW/zoom)/2,top_left.y+SIZE_ARROW/zoom), complex((top_left.x+bottom_right.x+SIZE_ARROW/zoom)/2,top_left.y));

	arrows[2].set_coords(complex(bottom_right.x,top_left.y+SIZE_ARROW/zoom),complex(bottom_right.x+SIZE_ARROW/zoom,top_left.y));

	arrows[3].set_coords(complex(bottom_right.x,(top_left.y+bottom_right.y+SIZE_ARROW/zoom)/2),complex(bottom_right.x+SIZE_ARROW/zoom,(top_left.y+bottom_right.y-SIZE_ARROW/zoom)/2));

	arrows[4].set_coords(complex(bottom_right.x,bottom_right.y),complex(bottom_right.x+SIZE_ARROW/zoom,bottom_right.y-SIZE_ARROW/zoom));

	arrows[5].set_coords(complex((top_left.x+bottom_right.x-SIZE_ARROW/zoom)/2,bottom_right.y), complex((top_left.x+bottom_right.x+SIZE_ARROW/zoom)/2,bottom_right.y-SIZE_ARROW/zoom));

	arrows[6].set_coords(complex(top_left.x-SIZE_ARROW/zoom,bottom_right.y),complex(top_left.x,bottom_right.y-SIZE_ARROW/zoom));

	arrows[7].set_coords(complex(top_left.x-SIZE_ARROW/zoom,(top_left.y+bottom_right.y+SIZE_ARROW/zoom)/2),complex(top_left.x,(top_left.y+bottom_right.y-SIZE_ARROW/zoom)/2));

	draw_arrows();
}

int selection_group::test_arrow_hit(complex screen_pos)
{
	for(int i=0;i<8;++i)
		if(arrows[i].test_hit(screen_pos))
			return i;
	return -1;
}

complex selection_group::get_scaling_middle(int ar)
{
	switch(ar)
	{
		case 0:	return complex(bottom_right.x, bottom_right.y);
		case 1:	return complex(bottom_right.x, bottom_right.y);
		case 2:	return complex(top_left.x, bottom_right.y);
		case 3:	return complex(top_left.x, bottom_right.y);
		case 4:	return complex(top_left.x, top_left.y);
		case 5:	return complex(bottom_right.x, top_left.y);
		case 6:	return complex(bottom_right.x, top_left.y);
		case 7:	return complex(bottom_right.x, bottom_right.y);
	}
}

complex selection_group::get_arrow_coords(int ar)
{
	switch(ar)
	{
		case 0:	return complex(top_left.x, top_left.y);
		case 1:	return complex((top_left.x+bottom_right.x)/2, top_left.y);
		case 2:	return complex(bottom_right.x, top_left.y);
		case 3:	return complex(bottom_right.x, (top_left.y+bottom_right.y)/2);
		case 4:	return complex(bottom_right.x, bottom_right.y);
		case 5:	return complex((top_left.x+bottom_right.x)/2, bottom_right.y);
		case 6:	return complex(top_left.x, bottom_right.y);
		case 7:	return complex(top_left.x, (top_left.y+bottom_right.y)/2);
	}
}

//////////////////////////////////////////////////////////////////


snapshot_rectangle::snapshot_rectangle()
{
	startx=156;
	starty=113;
	endx=868;
	endy=655;
}

void snapshot_rectangle::draw()
{
	glColor3f(1,1,1);
	glBegin(GL_LINE_LOOP);
		glVertex2f((startx*2*SCREEN_X/SCREEN_X_PIXELS)-SCREEN_X,(starty*2*SCREEN_Y/SCREEN_Y_PIXELS)-SCREEN_Y);
		glVertex2f((endx*2*SCREEN_X/SCREEN_X_PIXELS)-SCREEN_X,(starty*2*SCREEN_Y/SCREEN_Y_PIXELS)-SCREEN_Y);
		glVertex2f((endx*2*SCREEN_X/SCREEN_X_PIXELS)-SCREEN_X,(endy*2*SCREEN_Y/SCREEN_Y_PIXELS)-SCREEN_Y);
		glVertex2f((startx*2*SCREEN_X/SCREEN_X_PIXELS)-SCREEN_X,(endy*2*SCREEN_Y/SCREEN_Y_PIXELS)-SCREEN_Y);
	glEnd();
}

////////////////////////////////////////////////////////////

view::view()
{
	zoom=1;
	camera_position=complex(0,0);
	last_red=last_green=last_blue=0.5;
	cursor=complex(0,0);
}

view::~view()
{}

bool view::init()
{
	glShadeModel(GL_SMOOTH);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glFrontFace(GL_CCW);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); //max hint
	glMatrixMode(GL_PROJECTION); //switch to projection matrix
	glLoadIdentity(); //reset it
	gluPerspective(75,1.6,0.1,100); //fov, 4/3 screen dimension, near clip, far clip
	glMatrixMode(GL_MODELVIEW); //back to the model
	glClearColor(0,0,0.15,0); //background color - R,G,B,A
	return true;
}

void view::update()
{
	bool done=false;
	while(!done)
	{
		exaUSleep(20);
		exaUpdate(); //update cycle - wm and KB stuff
		if(exaIsKeyDown(EKEY_ESCAPE))
		{
			this->tidy_up();
			done=true; //exit on escape
		}
		if(exaIsKeyDown(EKEY_TAB))exaIconifyWindow(); //iconify=minimize
		if(!exaIsIconified()) render(); //render only if it will be seen
	}
}

void view::render()
{
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity(); //reset modelview
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT); // |depth buffer bit if we used depths

	glTranslatef(0,0,-19);
	glPointSize(3.0);

	if (exaIsMouseWheelUp()) zoom *= ZOOM_IN_MULTIPLIER;
	if (exaIsMouseWheelDown()) zoom *= ZOOM_OUT_MULTIPLIER;
	if (zoom < ZOOM_MIN) zoom = ZOOM_MIN;
	if (zoom > ZOOM_MAX) zoom = ZOOM_MAX;

	cursor/=zoom;
	cursor-=camera_position;
	glPushMatrix();
	bool make_image;
	if(!toolbar.is_making_snapshot())
	{
		glScalef(zoom, zoom, 0);
		glTranslatef(camera_position.x,camera_position.y,0);
	}
	else
		make_image=true;
	png_image_coords.draw();		//draw rectangle, which png picture is saved in
	if(exaIsMouseButtonHit(1))
	{
		from=from_pom=to=cursor;
		this->mouse_click(cursor);
	}
	if(exaIsMouseButtonDown(1))
	{
		to=cursor;
		if(from_pom!=to) mouse_drag(from,to);
	}
	else
	{
		this->selection.make_union(&(this->selection_pom));
		this->selection_pom.clean();
	}

	canvas.draw();
	if(!toolbar.is_making_snapshot() || make_image)
		toolbar.do_action(action,this);
	glPopMatrix();
	cursor+=camera_position;
	cursor*=zoom;

	this->draw_cursor();
	this->toolbar.draw_toolbar();
	this->color_toolbar.draw_toolbar();
	this->toolbar.check_button_tooltips(cursor);

	glFinish(); //wait till drawing finishes
	exaglSwapBuffers(); //bring it to foreground
}

void view::shutdown()
{
	exaShutdown();
}

void view::draw_cursor()
{
	int x,y;
	exaGetMouseMove (&x, &y);
	cursor += complex (x, -y) * 0.015;
	if(cursor.x > SCREEN_X) 
	{
		cursor.x = SCREEN_X;
		this->camera_position -= complex (x, -y) * 0.025;
	}
	if(cursor.x < -SCREEN_X)
	{
		cursor.x = -SCREEN_X;
		this->camera_position -= complex (x, -y) * 0.025;
	}
	if(cursor.y > SCREEN_Y)
	{
		cursor.y = SCREEN_Y;
		this->camera_position -= complex (x, -y) * 0.025;
	}
	if(cursor.y < -SCREEN_Y)
	{
		cursor.y = -SCREEN_Y;
		this->camera_position -= complex (x, -y) * 0.025;
	}
	glBegin(GL_POINTS);
		glColor3f(1,1,1);
		glVertex3f(cursor.x,cursor.y,0.1);
	glEnd();
}

void view::mouse_click(complex screen_pos)
{
	int pom=(this->toolbar.check_toolbar((screen_pos+camera_position)*zoom));
	if(pom!=(-1))		//toolbar clicked
	{
		if(pom!=toolbar.active_button) 
		{
			action.clear();
			if(pom==0)
				selection.clean();
			toolbar.active_button=pom;
			if(toolbar.active_button==1)
				toolbar.erase_resize_points_memory();
		}
	}
	else
	{
		action.push_back(screen_pos);
		if(toolbar.active_button==0)		//select button active
			arrow_persist_scaling=selection.test_arrow_hit(screen_pos);			//test whether scaling arrow was clicked and save the result
	}
}

void view::mouse_drag(complex from, complex to)
{
	if(toolbar.active_button!=0)
		return;

	list<objekt*> ob_uk_list;
	list<objekt*>::iterator it;

	objekt* ob=canvas.get_pointed_object(from_pom);
	if(ob || arrow_persist_scaling!=-1)
	{
		if(from==from_pom)  //moved for the first time - we have to save current position of objects (in selection) being moved
		{
			objekt* ob_uk;
			list<objekt*> ob_uk_list;
			selection.get_o(ob_uk_list);
			list<objekt*> ob_uk_list2;
			canvas.get_o(ob_uk_list2);

			selection.clean();
			list<objekt*>::iterator it;
			it=ob_uk_list.begin();
			while(it!=ob_uk_list.end())
			{
				ob_uk=(*it)->duplicate();		//delete "old" pointers on objects; and replace them by pointers on their exact copies
				replace(ob_uk_list2.begin(),ob_uk_list2.end(),*it,ob_uk);
				selection.add_obj(ob_uk);
				++it;
			}
			canvas.change(ob_uk_list2);
		}

		if(arrow_persist_scaling!=-1)		//arrow hit -> scale
		{
			if(selection.rotating)
			{
				complex tl,br,middle;
				selection.get_bounding_box(tl,br);
				middle=(tl+br)/2;
				complex c = (to - middle) / (from_pom - middle);
        		complex d = c.unit();
        		float angle = asinf (d.y);
        		if (d.x < 0) angle = -angle + M_PI;
				selection.rotate(middle,angle);
				from_pom=to;
				return;
			}

			complex middle=selection.get_scaling_middle(arrow_persist_scaling);		//point - centre for scaling
			complex ar=selection.get_arrow_coords(arrow_persist_scaling);
			switch(arrow_persist_scaling)
			{
				case 3: 	//right arrow dragged
					if(ar.x-(from_pom.x-to.x)<middle.x+RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					break;
				case 7:		//left arrow dragged
					if(ar.x-(from_pom.x-to.x)>middle.x-RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					break;
				case 1: 	//top arrow dragged
					if(ar.y-(from_pom.y-to.y)<middle.y+RESIZE_MIN)
						break;
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
				case 5:		//bottom arrow dragged
					if(ar.y-(from_pom.y-to.y)>middle.y-RESIZE_MIN)
						break;
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
				case 0:		//top_left arrow dragged
					if(ar.x-(from_pom.x-to.x)>middle.x-RESIZE_MIN || ar.y-(from_pom.y-to.y)<middle.y+RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
				case 4:		//bottom_right arrow dragged
					if(ar.x-(from_pom.x-to.x)<middle.x+RESIZE_MIN || ar.y-(from_pom.y-to.y)>middle.y-RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
				case 2:		//top_right arrow dragged
					if(ar.x-(from_pom.x-to.x)<middle.x+RESIZE_MIN || ar.y-(from_pom.y-to.y)<middle.y+RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
				case 6:		//bottom_left arrow dragged
					if(ar.x-(from_pom.x-to.x)>middle.x-RESIZE_MIN || ar.y-(from_pom.y-to.y)>middle.y-RESIZE_MIN)
						break;
					selection.scale_x(middle, 1+((to.x-from_pom.x)/(ar.x-middle.x)));
					selection.scale_y(middle, 1+((to.y-from_pom.y)/(ar.y-middle.y)));
					break;
			}
		}
		else	//object hit -> move
		{
			selection.move(to-from_pom); //simply move selection
		}
		from_pom=to;
	}
	else
	{
		glColor3f(0.5,0.5,0.5);
		glBegin(GL_LINE_LOOP);
		glVertex2f(from.x,from.y);
		glVertex2f(to.x,from.y);
		glVertex2f(to.x,to.y);
		glVertex2f(from.x,to.y);
		glEnd();
		complex tl, br;
		canvas.get_o(ob_uk_list);
		it=ob_uk_list.begin();
		while(it!=ob_uk_list.end())
		{
			(*it)->get_bounding_box(tl,br);
			if(((tl.x >= from.x) && (tl.y <= from.y) && (br.x <= to.x) && (br.y >= to.y)) ||
					((tl.x >= to.x) && (tl.y <= to.y) && (br.x <= from.x) && (br.y >= from.y)) ||
					((tl.x>=to.x) && (tl.y<=from.y) && (br.x<=from.x) && (br.y>=to.y)) ||
					((tl.x>=from.x) && (tl.y<=to.y) && (br.x<=to.x) && (br.y>=from.y)))
				selection_pom.add_obj(*it);	
			++it;
		}
		selection_pom.get_o(ob_uk_list);
		it=ob_uk_list.begin();
		while(it!=ob_uk_list.end())
		{
			(*it)->get_bounding_box(tl,br);
			if(!(((tl.x >= from.x) && (tl.y <= from.y) && (br.x <= to.x) && (br.y >= to.y)) ||
						((tl.x >= to.x) && (tl.y <= to.y) && (br.x <= from.x) && (br.y >= from.y)) ||
						((tl.x>=to.x) && (tl.y<=from.y) && (br.x<=from.x) && (br.y>=to.y)) ||
						((tl.x>=from.x) && (tl.y<=to.y) && (br.x<=to.x) && (br.y>=from.y))))
				selection_pom.rem_obj(*it);	
			++it;
		}
	}
}

void view::change_color(float r, float g, float b)
{
	if(r) last_red+=r;
	if(g) last_green+=g;
	if(b) last_blue+=b;
	if(last_red<0) last_red=0;
	if(last_red>1) last_red=1;
	if(last_green<0) last_green=0;
	if(last_green>1) last_green=1;
	if(last_blue<0) last_blue=0;
	if(last_blue>1) last_blue=1;
}

void view::change_color_absolutely(float r, float g, float b)
{
	last_red=r;
	last_green=g;
	last_blue=b;
	if(last_red<0) last_red=0;
	if(last_red>1) last_red=1;
	if(last_green<0) last_green=0;
	if(last_green>1) last_green=1;
	if(last_blue<0) last_blue=0;
	if(last_blue>1) last_blue=1;
}

void view::tidy_up() //uklid pred ukoncenim
{
	this->selection.clean();
	this->selection_pom.clean();
}
