#ifndef __EDITOR
#define __EDITOR
#include <list>
#include <vector>
#include "complex.h"
#include "buttons_editor.h"
#include "objects_editor.h"

/**trieda sluzi na komunikaciu a vsetky zmeny zoznamu objektov, ktore su prave vykreslovane.
 * je implementovana, aby sa starala o tzv. undo*/
class canvas_group
{
	/**pole objekt_group-ov, kde kazdy ma vlastne objekty*/
	objekt_group* can;
	/**urcuje aktivny objekt_group - prave vykreslovany*/
	int active;
	/**urcuje, kolko undo krokov sme spravili, aby sme rovnaky pocet mohli aj vratit undo*/
	int diff;
	/**urcuje pocet jednotlivych objekt_group-ov(=pocet undo)*/
	int size;
public:	
	canvas_group();
	~canvas_group();
	/**vykresli aktivny(aktualny) list objektov*/
	void draw();
	/**zmeni list objektov presne na ten, ktory ma zadany ako parameter*/
	void change(list<objekt*>&);
	/**vrati aktualne vykreslovany list objektov*/
	void get_o(list<objekt*>&);
	/**odstrani objekt z aktualneho listu*/
	void rem_obj(objekt*);
	/**prida objekt na koniec aktualneho listu*/
	void add_obj(objekt*);
	/**priblizi objekt o jeden krok dopredu, ak sa pred nim nejaky nachadza*/
	void raise_step(objekt*);
	/**priblizi objekt uplne dopredu, ak sa pred nim nejaky nachadza*/
	void raise_top(objekt*);
	/**oddiali objekt o jeden krok dozadu, ak sa za nim nejaky nachadza*/
	void lower_step(objekt*);
	/**oddiali uplne dozadu, ak sa za nim nejaky nachadza*/
	void lower_bottom(objekt*);
	/**uskutocni undo*/
	void backward();
	/**vrati naspat undo*/
	void forward();
	/**Vola sa pred tym, nez sa uskutocni nejaka zmena objektov. */
	void overwrite(int position);
	/**vrati objekt, ktory sa nachadza na zadanych suradniciach
	 * \note vola sa vtedy, ked klikneme na obrazovku, a chceme zistit ci sme tymto kliknutim oznacili nejaky objekt*/
	objekt* get_pointed_object(complex);
	/**vrati objekt, ktory sa nachadza na zadanych suradniciach, avsak musi to byt konkretny vykreslitelny objekt (nie objekt_group)
	 * \note vola sa vtedy, ak chceme zmenit polohu nejakeho bodu objektu typu poly_objekt(ciara, polygon, hviezdicka), preto potrebujeme konkretny objekt, nestaci pripadny objekt_group*/
	objekt* get_pointed_object_concrete(complex);
};

/**predstavuje posuvatka/rolovatka na kraji oznacenych objektov*/
class arrow		//arrows at the edge of selection in order to scale the selection
{
	/**suradnice laveho horneho rohu*/
	complex top_left;
	/**suradnice praveho dolneho rohu*/
	complex bottom_right;
public:
	/**nastavi polohu posuvatka/rolovatka*/
	void set_coords(complex tl, complex br);
	/**vykresli posuvatka/rolovatka
	 * param rotate ak je nastaveny, su kreslene rotovatka (inak posuvatka)*/
	void draw(bool rotate);  
	/**testuje, ci doslo ku kliknutiu na posuvatko/rolovatko*/
	bool test_hit(complex cursor);
};

/**trieda, ktora obsahuje odkazy na prave oznacene objekty*/
class selection_group : public objekt_group		//contains actually selected objects
{
	/**pole posuvatok/rolovatok*/
	arrow* arrows;
	/**vykresli posuvatka/rolovatka*/
	void draw_arrows();
public:
	/**indikuje, ze ci prave posuvame, alebo rotujeme oznacene objekty*/
	bool rotating;
	selection_group();
	~selection_group();
	/**vykreslime oznacenie objektu aj so sipkami*/
	void draw_selection(float);	
	/**otestujeme, ci posuvatka/rolovatka neboli potiahnute*/
	int test_arrow_hit(complex);	//test whether a user drags scaling arrows
	/**\return stred, okolo ktoreho sa budu oznacene objekty posuvat*/
	complex get_scaling_middle(int);	//returns centre around which we perform scaling
	/**\return suradnice ziadaneho posuvatka/rolovatka*/
	complex get_arrow_coords(int);
	/**\return multiplikator, kolkokrat sa bude selection zvacsovat/zmensovat*/
	float get_scaling_multiplier(int,complex);	//returns multiplicator for scaling selection
};

/**obdlnizk, ktory je stale zobrazovany a ked dame ulozit obrazok do png, tak sa nam ulozi prave vsetko, co sa nachadza vnutri tohto obdlznika*/
struct snapshot_rectangle
{
	/**x-ova suradnica pociatoceho bodu*/
	int startx;
	/**y-ova suradnica pociatoceho bodu*/
	int starty;
	/**x-ova suradnica koncoveho bodu*/
	int endx;
	/**y-ova suradnica koncoveho bodu*/
	int endy;
	snapshot_rectangle();
	/**vykresli obdlznik, vnutrajsok ktoreho bude v pripade vytvarania obrazka zachyteny*/
	void draw();
};

/**trieda view ma v podstate na starosti beh celeho programu, stara sa o poziciu "kamery" a "zoom", zaroven osetruje aj kliknutia mysi*/
class view
{
private:
	int arrow_persist_scaling;
public:
	/**udrzujeme si suradnice, kde bolo "kliknute", aby sme tak mohli vykonat operacie, vyzadujuce viac "kliknuti"*/
	vector<complex> action;
	/**urcuje ako sme si priblizili/oddialili vykreslovane objekty*/
	float zoom;
	/**urcuje cervenu zlozku farby, akou sa defaultne vykresluju nove objekty*/
	float last_red;
	/**urcuje zelenu zlozku farby, akou sa defaultne vykresluju nove objekty*/
	float last_green;
	/**urcuje modru zlozku farby, akou sa defaultne vykresluju nove objekty*/
	float last_blue;
	view();
	~view();
	/**urcuje pociatocnu polohu, odkial sa "tahalo" myskou*/
	complex from; 
	/**urcuje koncovu polohu, po kadial sa doposial "tahalo" myskou*/
	complex to;
	/**urcuje polohu, odkial sa "tahalo" myskou, ktora este nebola spracovana*/
	complex from_pom;
	/**aktualne suradnice kurzoru*/
	complex cursor;
	/**urcuje, okolko sme sa posunuli od uvodnej pozicie*/
	complex camera_position;
	/**zdruzuje vsetky cudliky*/
	toolbarx toolbar;
	/**zdruzuje vsetky farebne cudliky - po ich kliknuti selection zmeni farbu*/
	color_toolbarx color_toolbar;
	/**obdlznik, ktory je ulozeny, ked si dame ulozit obrazok do png*/
	snapshot_rectangle png_image_coords;
	/**selection zdruzuje prave oznacene objekty*/
	selection_group selection;
	/***/
	objekt_group selection_pom;
	/**canvas zdruzuje vsetky objekty - implementuje takisto undo*/
	canvas_group canvas;

	/**inicializujeme opengl a podobne*/
	bool init();
	/**touto metodou vlastne zaciname editor, osetruje vypnutie celeho programu, ako aj jeho minimalizovanie*/
	void update();
	/**ukoncuje pracu s SDL, volana pred ukoncenim programu*/
	void shutdown();
	/**nastavuje farbu, ktorou sa defaultne farbia novovzniknute objekty
	 * \note meni farbu tak, ze sa k aktualnej farbe pripocitaju/odcitaju hodnoty parametrov*/
	void change_color(float r, float g, float b);
	/**nastavuje farbu, ktorou sa defaultne farbia novovzniknute objekty
	 * \note meni farbu tak, ze sa nastavi na prave tie hodnoty, ktore su v parametroch*/
	void change_color_absolutely(float r, float g, float b);
private:
	/**vykreslime kurzor*/
	void draw_cursor();
	/**osetruje kliknutie mysou niekde na obrazovke*/
	void mouse_click(complex screen_pos);	//bude volana po kliknuti mysou niekde v aplikacii
	/**osetruje potiahnutie mysou (pohyb mysou za staleho drzania)*/
	void mouse_drag(complex from, complex to);
	/**hlavna metoda, vo svojom tele vola metody na vykreslovanie objektov, na osetrenie prace s mysou a klavesami, na vykonanie akcie prislusnej k stlacenemu "cudliku"*/
	void render();
	/**uklid pred ukoncenim*/
	void tidy_up();
};

#endif
