#include "objects_editor.h"
#include <cmath>
#include <vector>
#include <algorithm>
#include <list>
#include <GL/gl.h>

#define RESIZE_POINT_SIZE 0.12
#define HIT_TOLERANCY 0.25

objekt::objekt(float r, float g, float b) : red(r), green(g), blue(b)
{
	top_left=bottom_right=complex(0,0);
}
objekt::objekt()
{
	top_left=bottom_right=complex(0,0);
	red=green=blue=0.5;
}

bool objekt::test_hit(complex cursor)
{
	check_bounding_box();
	if((cursor.x >= top_left.x) && (cursor.y <= top_left.y) && (cursor.x <= bottom_right.x) && (cursor.y >= bottom_right.y))
		return true;
	else 
		return false;
}

void objekt::get_bounding_box(complex& t_l, complex& b_r)
{
	check_bounding_box();
	t_l=top_left;
	b_r=bottom_right;
}

void objekt::move(complex m)
{
	float matrix[6];

	matrix[0]=1; matrix[1]=0; matrix[2]=m.x;
	matrix[3]=0; matrix[4]=1; matrix[5]=m.y;

	mult_matrix(matrix);
}
void objekt::scale(complex stred, float x)
{
	float matrix[6];

	matrix[0]=x; matrix[1]=0; matrix[2]=stred.x*(1-x);
	matrix[3]=0; matrix[4]=x; matrix[5]=stred.y*(1-x);

	mult_matrix(matrix);
}
void objekt::scale_x(complex stred, float x)
{
	float matrix[6];

	matrix[0]=x; matrix[1]=0; matrix[2]=stred.x*(1-x);
	matrix[3]=0; matrix[4]=1; matrix[5]=0;

	mult_matrix(matrix);
}
void objekt::scale_y(complex stred, float x)
{
	float matrix[6];

	matrix[0]=1; matrix[1]=0; matrix[2]=0;
	matrix[3]=0; matrix[4]=x; matrix[5]=stred.y*(1-x);

	mult_matrix(matrix);
}
void objekt::rotate(complex stred, float uhol)
{
	float matrix[6];
	float kosinus=cos(uhol);
	float sinus=sin(uhol);
	if(kosinus<0.00000001 && kosinus>-0.00000001) kosinus=0;	
	if(sinus<0.00000001 && sinus>-0.00000001) sinus=0;	

	matrix[0]=kosinus; matrix[1]=-sinus; 
	matrix[2]=(-stred.x*kosinus) + (stred.y*sinus) + (stred.x);
	matrix[3]=sinus; matrix[4]=kosinus; 
	matrix[5]=(-stred.x*sinus) - (stred.y*kosinus) + (stred.y);

	mult_matrix(matrix);
}

void objekt::mult_matrix_one(float matrix[6],complex& c)
{
	float m;
	m=matrix[0]*c.x + matrix[1]*c.y + matrix[2];
	c.y=matrix[3]*c.x + matrix[4]*c.y + matrix[5];
	c.x=m;
}

void objekt::change_color(float r, float g, float b)
{
	if(r) red+=r;
	if(g) green+=g;
	if(b) blue+=b;
	if(red<0) red=0;
	if(red>1) red=1;
	if(green<0) green=0;
	if(green>1) green=1;
	if(blue<0) blue=0;
	if(blue>1) blue=1;
}

void objekt::change_color_absolutely(float r, float g, float b)
{
	red=r;
	green=g;
	blue=b;
	if(red<0) red=0;
	if(red>1) red=1;
	if(green<0) green=0;
	if(green>1) green=1;
	if(blue<0) blue=0;
	if(blue>1) blue=1;
}

//////////////////////////////////////////////////////////////////


objekt* objekt_group::duplicate()
{
	objekt_group *ob=new objekt_group;
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{
		ob->add_obj((*it)->duplicate());
		++it;
	}
	return ob;
}

objekt* objekt_group::duplicate(poly_objekt** ob)
{
	objekt_group *ob_g=new objekt_group;
	objekt *obj;
	objekt_group *obj_g;
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{
		obj_g=dynamic_cast<objekt_group*> (*it);
		if(obj_g)
			obj=obj_g->duplicate(ob);
		else
			obj=(*it)->duplicate();

		if((*it)==(*ob))
		{
			*ob=dynamic_cast<poly_objekt*> (obj);
		}

		ob_g->add_obj(obj);
		++it;
	}
	return ob_g;
}

void objekt_group::check_bounding_box()
{
	complex tl,br,tl2,br2;
	list<objekt*>::iterator it=o.begin();
	(*it)->get_bounding_box(tl,br);
	(*it)->get_bounding_box(tl2,br2);
	
	while(it!=o.end())
	{
		(*it)->get_bounding_box(tl2,br2);
		
		if(tl2.x < tl.x) tl.x=tl2.x;
		if(tl2.y > tl.y) tl.y=tl2.y;
		if(br2.x > br.x) br.x=br2.x;
		if(br2.y < br.y) br.y=br2.y;
		++it;
	}
	if(tl!=top_left)
		top_left=tl;
	if(br!=bottom_right)
		bottom_right=br;
}

void objekt_group::draw()
{
	list<objekt*>::iterator it=o.begin();
	while(it!=o.end())
	{
		(*it)->draw();
		++it;
	}
}

bool objekt_group::test_hit(complex cursor)
{
	list<objekt*>::iterator it=o.begin();
	
	while(it!=o.end())
	{
		if((*it)->test_hit(cursor)) return true;
		++it;
	}
	return false;
}

objekt* objekt_group::test_hit_concrete(complex cursor)
{
	list<objekt*>::iterator it=o.end();
	
	while(it!=o.begin())
	{
		--it;
		if((*it)->test_hit(cursor))
		{
			objekt_group* og=dynamic_cast<objekt_group*> (*it);
			if(!og)
			{
				return (*it);
			}
			else
			{
				return og->test_hit_concrete(cursor);
			}
		}
	}
	return 0;
}

void objekt_group::mult_matrix(float matrix[6])
{
	list<objekt*>::iterator it=o.begin();
	
	while(it!=o.end())
	{
		(*it)->mult_matrix(matrix);
		++it;
	}
}

void objekt_group::clean()
{
	o.clear();
}

void objekt_group::add_obj(objekt* pointer)
{
	list<objekt*>::iterator it=o.begin();
	while(it!=o.end())
	{
		if(pointer==*it) break;
		++it;
	}
	if(it==o.end()) o.push_back(pointer);
}

void objekt_group::rem_obj(objekt* pointer)
{
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{
		if(pointer==*it)
		{
			o.erase(it);
			return;
		}
		++it;
	}
}

void objekt_group::get_o(list<objekt*>& ob_list)
{
	ob_list=o;
}

void objekt_group::put_o(list<objekt*>& ob_list)
{
	o=ob_list;
}

void objekt_group::change_color(float r, float g, float b)
{
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{
		(*it)->change_color(r,g,b);
		++it;
	}
}

void objekt_group::change_color_absolutely(float r, float g, float b)
{
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{
		(*it)->change_color_absolutely(r,g,b);
		++it;
	}
}

void objekt_group::make_union(objekt_group* og)
{
	list<objekt*>::iterator it=og->o.begin();

	while(it!=og->o.end())
	{
		if(find(this->o.begin(),this->o.end(),*it)==this->o.end())
			this->add_obj(*it);
		++it;
	}
}

bool overlap(complex& tl1,complex& br1,complex& tl2,complex& br2)
{
	if((br1.x<=tl2.x) || (tl1.x>=br2.x) || (tl1.y<=br2.y) || (br1.y>=tl2.y))
		return false;
	return true;
}

void objekt_group::raise_step(objekt* p_o, bool ukoncit)
{
	list<objekt*>::iterator it=o.begin();
	bool slobodno=false;
	complex tl1,br1,tl2,br2;

	while(it!=o.end())
	{	
		if(*it==p_o)
		{
			slobodno=true;
			++it;
			continue;
		}
		if(!slobodno)
		{
			++it;
			continue;
		}

		(*it)->get_bounding_box(tl1,br1);
		p_o->get_bounding_box(tl2,br2);
		if(overlap(tl1,br1,tl2,br2))
		{
			o.remove(p_o);
			++it;
			o.insert(it,p_o);
			--it;
			if(ukoncit)
				return;
		}
		++it;
	}
}

void objekt_group::raise_top(objekt* p_o)
{
	this->raise_step(p_o,false);
}

void objekt_group::lower_step(objekt* p_o,bool ukoncit)
{
	list<objekt*>::iterator it=o.end();
	complex tl1,br1,tl2,br2;
	bool slobodno=false;

	while(it!=o.begin())
	{	
		--it;
		if(*it==p_o)
		{
			slobodno=true;
			continue;
		}
		if(!slobodno)
			continue;

		(*it)->get_bounding_box(tl1,br1);
		p_o->get_bounding_box(tl2,br2);
		if(overlap(tl1,br1,tl2,br2))
		{
			o.remove(p_o);
			o.insert(it,p_o);
			if(ukoncit)
				return;
		}
	}
}

void objekt_group::lower_bottom(objekt* p_o)
{
	this->lower_step(p_o,false);
}

objekt_group::~objekt_group()
{
	list<objekt*>::iterator it=o.begin();

	while(it!=o.end())
	{	
		delete (*it);
		++it;
	}
}

bool objekt_group::member(objekt* ob)
{
	list<objekt*>::iterator it=o.begin();
	objekt_group* o_g;

	while(it!=o.end())
	{	
		if((*it)==ob)
			return true;
		++it;
	}
	return false;
}

//////////////////////////////////////////////////////////////////

poly_objekt::poly_objekt(float r, float g, float b) : objekt(r,g,b)
{}

void poly_objekt::check_bounding_box()
{
	vector<complex>::iterator it=points.begin();
	complex t_l(it->x,it->y),b_r(it->x,it->y);
	++it;

	while(it!=points.end())
	{
		if(it->x > b_r.x) b_r.x=it->x;
		if(it->x < t_l.x) t_l.x=it->x;
		if(it->y < b_r.y) b_r.y=it->y;
		if(it->y > t_l.y) t_l.y=it->y;
		++it;
	}
	if(t_l!=top_left)
		top_left=t_l;
	if(b_r!=bottom_right)
		bottom_right=b_r;
}

void poly_objekt::mult_matrix(float matrix[6])
{
	vector<complex>::iterator it=points.begin();
	
	while(it!=points.end())
	{
		mult_matrix_one(matrix,*it);
		++it;
	}
}

bool poly_objekt::test_hit_drag_points(complex screen_pos,complex** p,float zoom)
{
	vector<complex>::iterator it=points.begin();
	
	while(it!=points.end())
	{
		if(((*it)-screen_pos).length()<(HIT_TOLERANCY/zoom))
		{
			*p=&(*it);
			return true;
		}
		++it;
	}
	return false;
}

void poly_objekt::move_point(complex* point, complex m)
{
	float matrix[6];

	matrix[0]=1; matrix[1]=0; matrix[2]=m.x;
	matrix[3]=0; matrix[4]=1; matrix[5]=m.y;
	mult_matrix_one(matrix,*point);
}

void poly_objekt::draw_resizeable_points(float zoom)
{
	vector<complex>::iterator it=points.begin();
	while(it!=points.end())
	{
		glBegin(GL_QUADS);
			glColor3f(0,0,0);
			glVertex2f(it->x-(RESIZE_POINT_SIZE/zoom),it->y-(RESIZE_POINT_SIZE/zoom));
			glVertex2f(it->x-(RESIZE_POINT_SIZE/zoom),it->y+(RESIZE_POINT_SIZE/zoom));
			glVertex2f(it->x+(RESIZE_POINT_SIZE/zoom),it->y+(RESIZE_POINT_SIZE/zoom));
			glVertex2f(it->x+(RESIZE_POINT_SIZE/zoom),it->y-(RESIZE_POINT_SIZE/zoom));
		glEnd();
		glBegin(GL_POINTS);
			glPointSize(2.5);
			glColor3f(1,0,0);
			glVertex2f(it->x,it->y);
		glEnd();
		++it;
	}
	glPointSize(3.0);
}

////////////////////////////////////////////////////////////


polygon::polygon(vector<complex>& p, float r, float g, float b) : poly_objekt(r,g,b)
{
	points=p;
}

objekt* polygon::duplicate()
{
	objekt *ob=new polygon(points,red,green,blue);
	return ob;
}

void polygon::draw()
{
	vector<complex>::iterator it=points.begin();
	glColor3f(red,green,blue);
	glBegin(GL_POLYGON);
		while(it!=points.end())
		{
			glVertex2f(it->x,it->y);
			++it;
		}
	glEnd();
}

////////////////////////////////////////////////////////////

line::line(vector<complex>& p, float r, float g, float b) : poly_objekt(r,g,b)
{
	points=p;
}

objekt* line::duplicate()
{
	objekt *ob=new line(points,red,green,blue);
	return ob;
}

void line::draw()
{
	vector<complex>::iterator it=points.begin();
	glColor3f(red,green,blue);
	glBegin(GL_LINE_STRIP);
		while(it!=points.end())
		{
			glVertex2f(it->x,it->y);
			++it;
		}
	glEnd();
}

////////////////////////////////////////////////////////////

rectangle::rectangle(vector<complex>& p, float r, float g, float b) : poly_objekt(r,g,b)
{
	points=p;
}

objekt* rectangle::duplicate()
{
	objekt *ob=new rectangle(points,red,green,blue);
	return ob;
}

void rectangle::draw()
{
	vector<complex>::iterator it=points.begin();
	glColor3f(red,green,blue);
	glBegin(GL_QUADS);
		while(it!=points.end())
		{
			glVertex2f(it->x,it->y);
			++it;
		}
	glEnd();
}

////////////////////////////////////////////////////////////

ellipse::ellipse(vector<complex>& p, float r, float g, float b) : poly_objekt(r,g,b)
{
	points=p;
}

objekt* ellipse::duplicate()
{
	objekt *ob=new ellipse(points,red,green,blue);
	return ob;
}

void ellipse::draw()
{
	vector<complex>::iterator it=points.begin();
	glColor3f(red,green,blue);
	glBegin(GL_POLYGON);
		while(it!=points.end())
		{
			glVertex2f(it->x,it->y);
			++it;
		}
	glEnd();
}

bool ellipse::test_hit_drag_points(complex screen_pos,complex** p,float zoom)
{
	return false;
}

void ellipse::move_point(complex* point, complex m)
{}

void ellipse::draw_resizeable_points(float zoom)
{}

////////////////////////////////////////////////////////////


star::star(vector<complex>& p, float r, float g, float b) : poly_objekt(r,g,b)
{
	points=p;
}

objekt* star::duplicate()
{
	objekt *ob=new star(points,red,green,blue);
	return ob;
}

void star::draw()
{
	vector<complex>::iterator it=points.begin();
	vector<complex>::iterator it_middle=points.begin();
	glColor3f(red,green,blue);
	glBegin(GL_TRIANGLES);
	++it;
		while(it!=points.end())
		{
			glVertex2f(it->x,it->y);
			++it;
			if(it==points.end())
			{
				glVertex2f((++points.begin())->x,(++points.begin())->y);
			}
			else
			{
			glVertex2f(it->x,it->y);
			}
			glVertex2f(it_middle->x,it_middle->y);
		}
	glEnd();
}

bool star::test_hit_drag_points(complex screen_pos,complex** p,float zoom)
{
	vector<complex>::iterator it=points.begin();
	++it;
	while(it!=points.end())
	{
		if(((*it)-screen_pos).length()<HIT_TOLERANCY/zoom)
		{
			*p=&(*it);
			return true;
		}
		++it;
	}
	return false;
}

void star::move_point(complex* point, complex m)
{
	float matrix[6];

	vector<complex>::iterator it=points.begin();
	++it;
	bool mod2=false;
	vector<complex>::iterator middle=points.begin();
	while(it!=points.end())
	{
		
		if(&(*it)==point)
			break;
		mod2=!mod2;
		++it;
	}
	
	if(it==points.end())
		return;

	int j;
	for(it=points.begin(), ++it, j=1 ; it!=points.end(); ++j, ++it)
	{
		if(mod2)
		{
			if(j%2)
				continue;
		}
		else
		{
			if(!(j%2))
				continue;
		}

		complex c = (*point - *middle) / (*it - *middle);
        complex d = c.unit();
        float angle = asinf (d.y);
        if (d.x < 0) angle = -angle + M_PI;
		angle=-angle;

		float tx = (m.x * cosf(angle)) - (m.y * sinf(angle));
		float ty = (m.x * sinf(angle)) + (m.y * cosf(angle));
		matrix[0]=1; matrix[1]=0; matrix[2]=tx;
		matrix[3]=0; matrix[4]=1; matrix[5]=ty;

		mult_matrix_one(matrix,*it);
	}
}

void star::draw_resizeable_points(float zoom)
{
	vector<complex>::iterator it=points.begin();
	++it;
	while(it!=points.end())
	{
		glBegin(GL_QUADS);
			glColor3f(0,0,0);
			glVertex2f(it->x-(0.12/zoom),it->y-(0.12/zoom));
			glVertex2f(it->x-(0.12/zoom),it->y+(0.12/zoom));
			glVertex2f(it->x+(0.12/zoom),it->y+(0.12/zoom));
			glVertex2f(it->x+(0.12/zoom),it->y-(0.12/zoom));
		glEnd();
		glBegin(GL_POINTS);
			glPointSize(2.5);
			glColor3f(1,0,0);
			glVertex2f(it->x,it->y);
		glEnd();
		++it;
	}
	glPointSize(3.0);
}

////////////////////////////////////////////////////////////

objekt* circle::duplicate()
{
	objekt *ob=new circle(middle,radius,red,green,blue);
	return ob;
}

void circle::check_bounding_box()
{
	float rad=(middle-radius).length();

	top_left=complex(middle.x-rad,middle.y+rad);
	bottom_right=complex(middle.x+rad,middle.y-rad);
}

void circle::mult_matrix(float matrix[6])
{
	mult_matrix_one(matrix,middle);
	mult_matrix_one(matrix,radius);
}

circle::circle(complex mid, complex rad, float r, float g, float b) : middle(mid), radius(rad), objekt(r,g,b)
{}

void circle::draw()
{
	float rad=(middle-radius).length();

	glColor3f(red,green,blue);
	glBegin(GL_POLYGON);
		for(int i=0; i <= 360; i++)
		glVertex2f(middle.x+(cos(i*M_PI/180)*rad), middle.y+(sin(i*M_PI/180)*rad));
	glEnd();
}
