#ifndef _OBJECTS_EDITOR
#define _OBJECTS_EDITOR

#include <list>
#include <vector>
#include "complex.h"
using namespace std;

class poly_objekt;

/**ciste virtualna trieda, ktora predstavuje suhrne vlastnosti vsetkych objektov*/
class objekt
{
private:
	/**vypocita lavy horny bod a pravy dolny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	virtual void check_bounding_box()=0;
protected:
	/**lavy horny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	complex top_left;
	/**pravt dolny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	complex bottom_right;
	/**cervena zlozka farby, ktorej je objekt*/
	float red;
	/**zelena zlozka farby, ktorej je objekt*/
	float green;
	/**modra zlozka farby, ktorej je objekt*/
	float blue;
public:
	objekt(float r, float g, float b);
	objekt();
	virtual ~objekt() {}
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate()=0;
	/**vykresli sa*/
	virtual void draw()=0;  
	/**otestuje, ci doslo k oznaceniu objektu
	 * \param cursor predstavuje suradnice, kde doslo ku kliknutiu*/
	virtual bool test_hit(complex cursor);	//testujem, ci doslo k oznaceniu objektu
	/**ziskame obdlznik(bounding box), v kt. sa nachadza tento objekt*/
	void get_bounding_box(complex& top_left, complex& bottom_right);
	/**rotujeme objekt
	 * \param stred stred, okolo ktoreho budeme rotovat
	 * \param uloh uhol, o ktory budeme rotovat*/
	void rotate(complex stred, float uhol);	//rotujeme o uhol okolo stredu
	/**zvacsujeme/zmensuje objekt
	 * \param stred stred, okolo ktoreho budeme zmensovat/zvacsovat
	 * \param x hodnota, kolkokrat budeme zmensovat/zvacsovat*/
	void scale(complex stred, float x);	//objekt zvacsime x-krat
	/**zvacsujeme/zmensuje objekt len vodorovne (menime len x-ove suradnice*/
	void scale_x(complex stred, float x);	//objekt zvacsime x-krat -len vodorovne
	/**zvacsujeme/zmensuje objekt len vodorovne (menime len x-ove suradnice*/
	void scale_y(complex stred, float x);	//objekt zvacsime x-kra -len zvisle
	/**posunie objekt*/
	void move(complex m); //posunieme
	/**nasobenie suradnic maticou
	 * \param c nasobene suradnice
	 * \param matrix matica, ktorou nasobime (ina pre posun, rotaciu aj zmensovanie/zvacsovanie*/
	void mult_matrix_one(float matrix[6],complex& c); //samotne prevedenie rotate, scale a move
	/**rotate, scale a move v podobe nasobenia matic, volane konkretnymi triedami(bude volat mult_matrix_one(float[6],complex&))
	 * \note odvodene triedy objektov si uz sami zvolia, ktore svoje body chcu nasobit maticou*/
	virtual void mult_matrix(float[6])=0; 
	/**zmeni farbu o dane hodnoty
	 * \note k aktualnym farbam pripocita/odcita hodnoty parametrov*/
	virtual void change_color(float r, float g, float b);
	/**zmeni farbu na dane hodnoty*/
	virtual void change_color_absolutely(float r, float g, float b);
};

/**logicky objekt, ktore v sebe zdruzuje ine objekty*/
class objekt_group : public objekt	//skupina objektov
{
	/**uchovavame si jednotlive objekty, kt. boli zjednotene do jedneho spolocneho(vystupuju ako jediny), zaroven usporiadanie v list-e urcuje, v akom poradi budu objekty vykreslovane*/
	list<objekt*> o;
	/**rotate, scale a move v podobe nasobenia matic, volane konkretnymi triedami(bude volat mult_matrix_one(float[6],complex&))
	 * \note odvodene triedy objektov si uz sami zvolia, ktore svoje body chcu nasobit maticou*/
	virtual void mult_matrix(float[6]);
public:
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**metoda zduplikuje seba sameho, avsak okrem toho este ak narazi (prehladavanim do hlbky) na objekt specifikovany parametrom, tak jeho kopiu priradi danemu parametru
	 * \note pouziva sa vtedy, ked posuvame bod nejakeho poly_objektu, avsak pri jeho kopii (musime kopirovat z dovodu undo) sa dotycny objekt zmeni, a my tymto sposobom (,ze po duplikacii dostaneme pointer na spravny objekt) posuvame bod uz "fyzicky" ineho, ale logicky rovnakeho poly_objekty*/
	objekt* duplicate(poly_objekt** ob);	//duplicate plus assign to ob a new pointer created as duplication of ob
	/**vykresli sa*/
	virtual void draw();
	/**otestuje, ci doslo k oznaceniu objektu
	 * \param cursor predstavuje suradnice, kde doslo ku kliknutiu*/
	virtual bool test_hit(complex cursor);
	/**otestuje, ci doslo k oznaceniu objektu
	 * \note pouziva sa, ked chcem posuvat bod nejakeho poly_objektu, preto potrebujem vratit konkretny objekt (a nie pripadny objekt_group)
	 * \param cursor predstavuje suradnice, kde doslo ku kliknutiu
	 * \return vrati ukazatel na konkretny jeden "viditelny" objekt*/
	objekt* test_hit_concrete(complex);
	/**pridame novy objekt*/
	void add_obj(objekt*);
	/**odoberieme specifikovany objekt;*/
	void rem_obj(objekt*);
	/**list objektov vyprazdni*/
	virtual void clean();
	/**vracia list objektov*/
	void get_o(list<objekt*>& ob_list);
	/**uklada list objektov*/
	void put_o(list<objekt*>& ob_list);
	/**zmeni farbu o dane hodnoty
	 * \note k aktualnym farbam pripocita/odcita hodnoty parametrov*/
	virtual void change_color(float r, float g, float b);
	/**zmeni farbu na dane hodnoty*/
	virtual void change_color_absolutely(float r, float g, float b);
	/**do svojho listu objektov prida list objektov ineho objekt_group specifikovaneho v parametru*/
	void make_union(objekt_group* og);
	/**vypocita lavy horny bod a pravy dolny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	virtual void check_bounding_box();	
	//objekt_group* get_closest_objekt_group(objekt*);	//vrati najblizsi nadriadeny objekt_group, ktora obsahuje hladany objekt
	/**presunie, vramci listu objektov, specifikovany objekt uplne dopredu*/
	void raise_top(objekt* p_o);  //
	/**presunie, vramci listu objektov, specifikovany objekt uplne dozadu*/
	void lower_bottom(objekt* p_o);
	/**presunie, vramci listu objektov, specifikovany objekt o jeden krok dopredu*/
	void raise_step(objekt* p_o,bool ukoncit=true); //presunie o 1 "viditelny" presun dopredu/dozadu
	/**presunie, vramci listu objektov, specifikovany objekt o jeden krok dozadu*/
	void lower_step(objekt* p_o,bool ukoncit=true);
	/**zisti, ci dany objekt je umiestneny vramci objekt_group*/
	bool member(objekt*); 
	/**odalokuje vsetky objekty, ktore si drzi vo svojom list-e*/
	~objekt_group();
};

/**objekty, kt. si musia zapamatat viacero bodov*/
class poly_objekt : public objekt
{
	/**vypocita lavy horny bod a pravy dolny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	virtual void check_bounding_box();	
	/**rotate, scale a move v podobe nasobenia matic, volane konkretnymi triedami(bude volat mult_matrix_one(float[6],complex&))
	 * \note odvodene triedy objektov si uz sami zvolia, ktore svoje body chcu nasobit maticou*/
	virtual void mult_matrix(float[6]);
protected:
	/**pole suradnic jednotlivych bodov, z ktorych sa sklada objekt*/
	vector<complex> points;
public:
	poly_objekt(float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate()=0;
	/**vykresli sa*/
	virtual void draw()=0;  
	/**testujeme, ze ci nedoslo ku kliknutiu na niektory z bodov poly_objektu
	 * \param screen_pos misto, kde dosla ku kliknutiu na obrazovke
	 * \param p ak bolo kliknute na niektory z bodov, tak metoda ho vrati tymto parametrom
	 * \param zoom tolerancia v kliknuti na bod musi zohladnovat aj zoom*/
	virtual bool test_hit_drag_points(complex screen_pos, complex** p,float zoom);
	/**pohyna niektorym z bodov poly_objektu*/
	virtual void move_point(complex* point,complex m);
	/**vykresli body, ktore je mozne posuvat*/
	virtual void draw_resizeable_points(float zoom);
};

/**trieda, ktora reprezentuje mnohouholnik*/
class polygon : public poly_objekt
{
public:
	polygon(vector<complex>& p, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
};

/**trieda, ktora reprezentuje obdlznik*/
class rectangle : public poly_objekt
{
public:
	rectangle(vector<complex>& p, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
};

/**trieda, ktora reprezentuje ciaru*/
class line : public poly_objekt
{
public:
	line(vector<complex>& p, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
};

/**trieda, ktora reprezentuje elipsu*/
class ellipse : public poly_objekt
{
public:
	ellipse(vector<complex>& p, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
	virtual bool test_hit_drag_points(complex screen_pos, complex** p,float zoom);
	virtual void move_point(complex* point,complex m);
	virtual void draw_resizeable_points(float zoom);
};

/**trieda, ktora reprezentuje kruh*/
class ellipse_circle : public ellipse
{
public:
	ellipse_circle(vector<complex>& p, float r, float g, float b) : ellipse(p,r,g,b) {}
};

/**trieda, ktora reprezentuje hviezdicku*/
class star : public poly_objekt
{
public:
	star(vector<complex>& p, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
	/**testujeme, ze ci nedoslo ku kliknutiu na niektory z bodov poly_objektu
	 * \param screen_pos misto, kde dosla ku kliknutiu na obrazovke
	 * \param p ak bolo kliknute na niektory z bodov, tak metoda ho vrati tymto parametrom
	 * \param zoom tolerancia v kliknuti na bod musi zohladnovat aj zoom*/
	virtual bool test_hit_drag_points(complex screen_pos, complex** p,float zoom);
	/**pohyna niektorym z bodov poly_objektu*/
	virtual void move_point(complex* point,complex m);
	/**vykresli body, ktore je mozne posuvat*/
	virtual void draw_resizeable_points(float zoom);
};

class circle : public objekt
{
	complex middle, radius;
	/**vypocita lavy horny bod a pravy dolny bod obdlzniku(bounding boxu), v ktorom sa nachadza objekt*/
	virtual void check_bounding_box();	
	/**rotate, scale a move v podobe nasobenia matic, volane konkretnymi triedami(bude volat mult_matrix_one(float[6],complex&))
	 * \note odvodene triedy objektov si uz sami zvolia, ktore svoje body chcu nasobit maticou*/
	virtual void mult_matrix(float[6]);
public:
	circle(complex mid, complex rad, float r, float g, float b);
	/**zduplikuje seba sameho
	 * \return ukazatel na novovytvorenu kopiu seba sameho*/
	virtual objekt* duplicate();
	/**vykresli sa*/
	virtual void draw();  
};

#endif
